/*
  Control_TC

  Based on PID Relay example and TC Shield examples.
  PID Relay example is weakly commented.  Analog example
  is much better.  Minimum time is ok?
  
  // I had to disable error checking to insert a negative Kd  
  // (a negative derivative coefficient).  I am sure I want a 
  // negative derivative coefficient because the system is mostly
  // off (not working hard).  If it stops working, the temperature
  // drops very slowly.  Thus it's never important to have a 
  // positive derivative coefficient.  
  // Would I instead use an integral coefficient to avoid changing
  // the PID source??   Then most of the "turn on" behavior is due 
  // to the cumulative lack of sufficient temperature.
  // the only problem is that I don't know how this parameter 
  // behaves over time -- how long before integral is flushed?
  // I guess it's the "Window Size" parameter -- 5000 millis.
  
  This work is licensed under a Creative Commons Attribution-ShareAlike 3.0 Unported License.
  http://creativecommons.org/licenses/by-sa/3.0/
*/

#include <MAX31855.h>
#include <PID_v1.h>
#define RELAY 7
#define minimumRelayTime 750 // don't waste throws of the switch
#define initialSetpoint 120 // debugging
#define oneminute 60000 // 60,000 millis per minute

//Define Variables we'll be connecting to
double Setpoint, Output;
double tempTC, tempCJC;
bool faultOpen, faultShortGND, faultShortVCC, x;

//Specify the links and initial tuning parameters
//Define the aggressive and conservative Tuning Parameters
double aggKp=400, aggKi=0.1, aggKd=-50;
// Each degree centigrade as one point towards the decision
// to turn on or off.  Kp scales this up toward 5000.
double consKp=1, consKi=0.05, consKd=0;
PID myPID(&tempTC, &Output, &Setpoint, aggKp, aggKi, aggKd, DIRECT);

// Time window of 5000 milliseconds?
int WindowSize = 5000;
unsigned long windowStartTime; // each new increment

MAX31855 temp(13, 10, 12); // measure and fault pins

void setup() {
  Serial.begin(9600);
  windowStartTime = millis();
  pinMode(RELAY,OUTPUT);
  
  //initialize the variables we're linked to
  Setpoint = initialSetpoint;

  //tell the PID to range between 0 and the full window size
  myPID.SetOutputLimits(0, WindowSize);

  //turn the PID on
  myPID.SetMode(AUTOMATIC);
}

void loop() {
    x = temp.readMAX31855(&tempTC, &tempCJC, &faultOpen, &faultShortGND, &faultShortVCC);
    double gap = abs(Setpoint-tempTC); //distance away from setpoint
    
    myPID.Compute();
  
    /************************************************
     * turn the output pin on/off based on pid output
     ************************************************/
    unsigned long now = millis();
    Setpoint = initialSetpoint + double(now) / oneminute;
    if(now - windowStartTime>WindowSize)
    { //time to shift the Relay Window
        windowStartTime += WindowSize;
        reportResult(Setpoint,tempTC,tempCJC,faultOpen,faultShortGND,faultShortVCC,Output,WindowSize);
    }
    
    if(Output > now - windowStartTime)
    { // if we're in the "on" part of the duty cycle
        if(Output > minimumRelayTime)
        { // check to see if it's worth turning on.
            digitalWrite(RELAY,HIGH);
        }
        reportResult(Setpoint,tempTC,tempCJC,faultOpen,faultShortGND,faultShortVCC,Output,WindowSize);
        delay(minimumRelayTime);
        reportResult(Setpoint,tempTC,tempCJC,faultOpen,faultShortGND,faultShortVCC,Output,WindowSize);
    } else digitalWrite(RELAY,LOW);
}

void reportResult(double Setpoint, double tempTC, double tempCJC, 
                  bool faultOpen, bool faultShortGND, bool faultShortVCC,
                  double Output, int WindowSize)
{
      // Report during each new window (about 5000 milliseconds)
      Serial.print(Setpoint);
      Serial.print("\t");
      Serial.print(tempTC);
      Serial.print("\t");
      Serial.print(tempCJC);
      Serial.print("\t");
      Serial.print(faultOpen);
      Serial.print(faultShortGND);
      Serial.print(faultShortVCC); 
      Serial.print("\t"); 
      Serial.print(int(Output));
      Serial.print("\t");
      Serial.println(WindowSize);
}
